using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;
using System.IO;
using System.Windows.Forms;
using Powertek;
using Powertek.Imaging;

namespace ColorConverter
{   
  public static class Config
  {
    public static string AppCaption;
    public static string AppVersion;
    public static PtColorSpace SrcColorSpace = PtColorSpace.RGB;
    public static PtColorSpace DstColorSpace = PtColorSpace.I1I2I3;
    public static double SrcX = 0.0;
    public static double SrcY = 0.0;
    public static double SrcZ = 0.0;
    public static bool SmallWindow = true;

    // saves the config to the registry
    public static void RegIniRead()
    {
      int temp;

      try
      {

        string keyName = "HKEY_CURRENT_USER\\Software\\powertek\\" +
                         AppCaption + "\\" + AppVersion +
                         "\\settings";

        SrcX = (double)Convert.ToDouble((string)Registry.GetValue(keyName, "srcX", 0.0));
        SrcY = (double)Convert.ToDouble((string)Registry.GetValue(keyName, "srcY", 0.0));
        SrcZ = (double)Convert.ToDouble((string)Registry.GetValue(keyName, "srcZ", 0.0));

        temp = (int)Registry.GetValue(keyName, "srcColorSpace", 0);
        switch (temp)
        {
          case 0:
            SrcColorSpace = PtColorSpace.RGB;
            break;
          case 1:
            SrcColorSpace = PtColorSpace.CMY;
            break;
          case 2:
            SrcColorSpace = PtColorSpace.HSV;
            break;
          case 3:
            SrcColorSpace = PtColorSpace.HunterLAB;
            break;
          case 4:
            SrcColorSpace = PtColorSpace.I1I2I3;
            break;
          case 5:
            SrcColorSpace = PtColorSpace.LAB;
            break;
          case 6:
            SrcColorSpace = PtColorSpace.LUV;
            break;
          case 7:
            SrcColorSpace = PtColorSpace.XyY;
            break;
          case 8:
            SrcColorSpace = PtColorSpace.XYZ;
            break;
        }


        temp = (int)Registry.GetValue(keyName, "dstColorSpace", 4);
        switch (temp)
        {
          case 0:
            DstColorSpace = PtColorSpace.RGB;
            break;
          case 1:
            DstColorSpace = PtColorSpace.CMY;
            break;
          case 2:
            DstColorSpace = PtColorSpace.HSV;
            break;
          case 3:
            DstColorSpace = PtColorSpace.HunterLAB;
            break;
          case 4:
            DstColorSpace = PtColorSpace.I1I2I3;
            break;
          case 5:
            DstColorSpace = PtColorSpace.LAB;
            break;
          case 6:
            DstColorSpace = PtColorSpace.LUV;
            break;
          case 7:
            DstColorSpace = PtColorSpace.XyY;
            break;
          case 8:
            DstColorSpace = PtColorSpace.XYZ;
            break;
        }


        temp = (int)Registry.GetValue(keyName, "smallWindow", 1);
        if (temp == 1)
          SmallWindow = true;
        else
          SmallWindow = false;

      }
      catch
      {
      }
    }


    public static void RegIniSave()
    {
      try
      {

        string keyName = "HKEY_CURRENT_USER\\Software\\powertek\\" +
                         AppCaption + "\\" + AppVersion +
                         "\\settings";

        Registry.SetValue(keyName, "srcX", SrcX);
        Registry.SetValue(keyName, "srcY", SrcY);
        Registry.SetValue(keyName, "srcZ", SrcZ);
       
        switch (SrcColorSpace)
        {
          case PtColorSpace.RGB:
            Registry.SetValue(keyName, "srcColorSpace", 0);
            break;
          case PtColorSpace.CMY:
            Registry.SetValue(keyName, "srcColorSpace", 1);
            break;
          case PtColorSpace.HSV:
            Registry.SetValue(keyName, "srcColorSpace", 2);
            break;
          case PtColorSpace.HunterLAB:
            Registry.SetValue(keyName, "srcColorSpace", 3);
            break;
          case PtColorSpace.I1I2I3:
            Registry.SetValue(keyName, "srcColorSpace", 4);
            break;
          case PtColorSpace.LAB:
            Registry.SetValue(keyName, "srcColorSpace", 5);
            break;
          case PtColorSpace.LUV:
            Registry.SetValue(keyName, "srcColorSpace", 6);
            break;
          case PtColorSpace.XyY:
            Registry.SetValue(keyName, "srcColorSpace", 7);
            break;
          case PtColorSpace.XYZ:
            Registry.SetValue(keyName, "srcColorSpace", 8);
            break;
        }

        switch (DstColorSpace)
        {
          case PtColorSpace.RGB:
            Registry.SetValue(keyName, "dstColorSpace", 0);
            break;
          case PtColorSpace.CMY:
            Registry.SetValue(keyName, "dstColorSpace", 1);
            break;
          case PtColorSpace.HSV:
            Registry.SetValue(keyName, "dstColorSpace", 2);
            break;
          case PtColorSpace.HunterLAB:
            Registry.SetValue(keyName, "dstColorSpace", 3);
            break;
          case PtColorSpace.I1I2I3:
            Registry.SetValue(keyName, "dstColorSpace", 4);
            break;
          case PtColorSpace.LAB:
            Registry.SetValue(keyName, "dstColorSpace", 5);
            break;
          case PtColorSpace.LUV:
            Registry.SetValue(keyName, "dstColorSpace", 6);
            break;
          case PtColorSpace.XyY:
            Registry.SetValue(keyName, "dstColorSpace", 7);
            break;
          case PtColorSpace.XYZ:
            Registry.SetValue(keyName, "dstColorSpace", 8);
            break;
        }


        if (SmallWindow)
          Registry.SetValue(keyName, "smallWindow", 1);
        else
          Registry.SetValue(keyName, "smallWindow", 0);

      }
      catch
      {
      }

    }


  }
}
