using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Powertek;
using Powertek.Imaging;

namespace ColorConverter
{
  public partial class FormMaster : Form
  {
    private Bitmap _bmp;
    ToolTip toolTip_SrcX = new ToolTip();
    ToolTip toolTip_SrcY = new ToolTip();
    ToolTip toolTip_SrcZ = new ToolTip();

    private int _globalMouseX = 0;
    private int _globalMouseY = 0;

    private Bitmap _screenBmp;
    private Bitmap _pb1Bmp;
    private Color _screenColor = Color.Black;

    [DllImport("user32.dll")]
    static extern bool GetCursorPos(ref Point lpPoint);

    public FormMaster()
    {
      InitializeComponent();

      Config.AppCaption = "RGB Space Color Converter";
      Config.AppVersion = "015";
    }
     

    private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
    {
      // save the old values
      double oldSrcX = Config.SrcX;
      double oldSrcY = Config.SrcY;
      double oldSrcZ = Config.SrcZ;
      PtColorSpace oldColorSpace = Config.SrcColorSpace;

      // get the new ones
      float x = 0;
      float y = 0;
      float z = 0;

      try
      {
        x = (float)Convert.ToDouble(textBox_srcX.Text);
        y = (float)Convert.ToDouble(textBox_srcY.Text);
        z = (float)Convert.ToDouble(textBox_srcZ.Text);
      }
      catch
      {
        x = y = z = 0;
      }
      
      switch (comboBox1.SelectedIndex)
      {
        case 0:
          Config.SrcColorSpace = PtColorSpace.RGB;
          label_srcX.Text = "R [0..255]";
          label_srcY.Text = "G [0..255]";
          label_srcZ.Text = "B [0..255]";
          break;

        case 1:
          Config.SrcColorSpace = PtColorSpace.CMY;
          label_srcX.Text = "C [0.0..1.0]";
          label_srcY.Text = "M [0.0..1.0]";
          label_srcZ.Text = "Y [0.0..1.0]";
          break;

        case 2:
          Config.SrcColorSpace = PtColorSpace.HSV;
          label_srcX.Text = "H [0.0..360.0]";
          label_srcY.Text = "S [0.0..100.0]";
          label_srcZ.Text = "V [0.0..100.0]";
          break;

        case 3:
          Config.SrcColorSpace = PtColorSpace.HunterLAB;
          label_srcX.Text = "(H) L [0.0..100.0]";
          label_srcY.Text = "(H) a [-72.52..104.92]";
          label_srcZ.Text = "(H) b [-190.92..58.90]";
          break;

        case 4:
          Config.SrcColorSpace = PtColorSpace.I1I2I3;
          label_srcX.Text = "I1 [0..255]";
          label_srcY.Text = "I2 [-127.5..127.50]";
          label_srcZ.Text = "I3 [-127.5..127.50]";
          break;

        case 5:
          Config.SrcColorSpace = PtColorSpace.LAB;
          label_srcX.Text = "L [0.0..100.0]";
          label_srcY.Text = "a [-86.18..98.25]";
          label_srcZ.Text = "b [-107.86..98.48]";
          break;

        case 6:
          Config.SrcColorSpace = PtColorSpace.LUV;
          label_srcX.Text = "L [0.0..100.0]";
          label_srcY.Text = "u [-83.08..175.05]";
          label_srcZ.Text = "V [-134.11..107.40]";
          break;

        case 7:
          Config.SrcColorSpace = PtColorSpace.XyY;
          label_srcX.Text = "x [0.0..0.64]";
          label_srcY.Text = "y [0.0..0.60]";
          label_srcZ.Text = "Y [0.0..100.0]";
          break;

        case 8:
          Config.SrcColorSpace = PtColorSpace.XYZ; 
          label_srcX.Text = "X [0.0..95.05]";
          label_srcY.Text = "Y [0.0..100.0]";
          label_srcZ.Text = "Z [0.0..108.9]";
          break;
      }

      SetToolTipAndLabel();

      // convert the old input to the new, so the color will still be equal
      PtColorConversion.Convert(oldColorSpace, Config.SrcColorSpace,
                                oldSrcX, oldSrcY, oldSrcZ,
                                out Config.SrcX, out Config.SrcY, out Config.SrcZ);

      if (Config.SrcColorSpace == PtColorSpace.RGB)
      {
        textBox_srcX.Text = Config.SrcX.ToString("0");
        textBox_srcY.Text = Config.SrcY.ToString("0");
        textBox_srcZ.Text = Config.SrcZ.ToString("0");
      }
      else
      {
        textBox_srcX.Text = Config.SrcX.ToString("0.00");
        textBox_srcY.Text = Config.SrcY.ToString("0.00");
        textBox_srcZ.Text = Config.SrcZ.ToString("0.00");
      }

      ConvertColor();
    }

    private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
    {
      switch (comboBox2.SelectedIndex)
      {
        case 0:
          Config.DstColorSpace = PtColorSpace.RGB; 
          label_dstX.Text = "R:";
          label_dstY.Text = "G:";
          label_dstZ.Text = "B:";
          break;

        case 1:
          Config.DstColorSpace = PtColorSpace.CMY;
          label_dstX.Text = "C:";
          label_dstY.Text = "M:";
          label_dstZ.Text = "Y:";
          break;

        case 2:
          Config.DstColorSpace = PtColorSpace.HSV;
          label_dstX.Text = "H ():";
          label_dstY.Text = "S (%):";
          label_dstZ.Text = "V (%):"; 
          break;

        case 3:
          Config.DstColorSpace = PtColorSpace.HunterLAB;
          label_dstX.Text = "(H) L:";
          label_dstY.Text = "(H) a:";
          label_dstZ.Text = "(H) b:";
          break;

        case 4:
          Config.DstColorSpace = PtColorSpace.I1I2I3;
          label_dstX.Text = "I1:";
          label_dstY.Text = "I2:";
          label_dstZ.Text = "I3:";    
          break;

        case 5:
          Config.DstColorSpace = PtColorSpace.LAB;
          label_dstX.Text = "L:";
          label_dstY.Text = "a:";
          label_dstZ.Text = "b:";
          break;

        case 6:
          Config.DstColorSpace = PtColorSpace.LUV; 
          label_dstX.Text = "L:";
          label_dstY.Text = "u:";
          label_dstZ.Text = "V:";
          break;

        case 7:
          Config.DstColorSpace = PtColorSpace.XyY; 
          label_dstX.Text = "x:";
          label_dstY.Text = "y:";
          label_dstZ.Text = "Y:";
          break;

        case 8:
          Config.DstColorSpace = PtColorSpace.XYZ;
          label_dstX.Text = "X:";
          label_dstY.Text = "Y:";
          label_dstZ.Text = "Z:";
          break;
      }

      ConvertColor();
    }



    public void ConvertColor()
    {
      double outX, outY, outZ;
      double inX, inY, inZ;

      outX = outY = outZ = 0.0;

      if (textBox_srcX.Text == "")
        textBox_srcX.Text = "0";

      if (textBox_srcY.Text == "")
        textBox_srcY.Text = "0";

      if (textBox_srcZ.Text == "")
        textBox_srcZ.Text = "0";

      try
      {
        inX = Convert.ToDouble(textBox_srcX.Text);
        inY = Convert.ToDouble(textBox_srcY.Text);
        inZ = Convert.ToDouble(textBox_srcZ.Text);

        Config.SrcX = inX;
        Config.SrcY = inY;
        Config.SrcZ = inZ;
      }
      catch
      {
        return;
      }

      // first convert the current src Value into rgb, so we can paint the pb
      double r, g, b;

      PtColorConversion.Convert(Config.SrcColorSpace, PtColorSpace.RGB,
                                Config.SrcX, Config.SrcY, Config.SrcZ,
                                out r, out g, out b);

      PaintPB(PtMath.ClampToByte(r), PtMath.ClampToByte(g), PtMath.ClampToByte(b));

      PtColorConversion.Convert(Config.SrcColorSpace, Config.DstColorSpace,
                                Config.SrcX, Config.SrcY, Config.SrcZ,
                                out outX, out outY, out outZ);

      if (Config.DstColorSpace == PtColorSpace.RGB)
      {
        label1.Text = outX.ToString("0");
        label2.Text = outY.ToString("0");
        label3.Text = outZ.ToString("0");
      }
      else
      {
        label1.Text = outX.ToString("0.000");
        label2.Text = outY.ToString("0.000");
        label3.Text = outZ.ToString("0.000");
      }
    }

    private void PaintPB(Byte r, Byte g, Byte b)
    {
      Graphics gr = Graphics.FromImage(_bmp);

      try
      {
        gr.Clear(Color.FromArgb(r, g, b));
        pictureBox1.Image = _bmp;
      }
      catch
      {
      }

      gr.Dispose();

    }


    private void Form1_Shown(object sender, EventArgs e)
    {
      
      _bmp = new Bitmap(pictureBox1.Width, pictureBox1.Height);

      textBox_srcX.Text = Config.SrcX.ToString("0.000");
      textBox_srcY.Text = Config.SrcY.ToString("0.000");
      textBox_srcZ.Text = Config.SrcZ.ToString("0.000");

      toolTip_SrcX.AutoPopDelay = toolTip_SrcY.AutoPopDelay = toolTip_SrcZ.AutoPopDelay = 5000;
      toolTip_SrcX.InitialDelay = toolTip_SrcY.InitialDelay = toolTip_SrcZ.InitialDelay = 1000;
      toolTip_SrcX.ReshowDelay = toolTip_SrcY.ReshowDelay = toolTip_SrcZ.ReshowDelay = 1000;
      toolTip_SrcX.ShowAlways = toolTip_SrcY.ShowAlways = toolTip_SrcZ.ShowAlways = true;

      SetToolTipAndLabel();

      switch (Config.SrcColorSpace)
      {
        case PtColorSpace.RGB:
          comboBox1.SelectedIndex = 0;
          break;
        case PtColorSpace.CMY:
          comboBox1.SelectedIndex = 1;
          break;
        case PtColorSpace.HSV:
          comboBox1.SelectedIndex = 2;
          break;
        case PtColorSpace.HunterLAB:
          comboBox1.SelectedIndex = 3;
          break;
        case PtColorSpace.I1I2I3:
          comboBox1.SelectedIndex = 4;
          break;
        case PtColorSpace.LAB:
          comboBox1.SelectedIndex = 5;
          break;
        case PtColorSpace.LUV:
          comboBox1.SelectedIndex = 6;
          break;
        case PtColorSpace.XyY:
          comboBox1.SelectedIndex = 7;
          break;
        case PtColorSpace.XYZ:
          comboBox1.SelectedIndex = 8;
          break;
      }

      switch (Config.DstColorSpace)
      {
        case PtColorSpace.RGB:
          comboBox2.SelectedIndex = 0;
          break;
        case PtColorSpace.CMY:
          comboBox2.SelectedIndex = 1;
          break;
        case PtColorSpace.HSV:
          comboBox2.SelectedIndex = 2;
          break;
        case PtColorSpace.HunterLAB:
          comboBox2.SelectedIndex = 3;
          break;
        case PtColorSpace.I1I2I3:
          comboBox2.SelectedIndex = 4;
          break;
        case PtColorSpace.LAB:
          comboBox2.SelectedIndex = 5;
          break;
        case PtColorSpace.LUV:
          comboBox2.SelectedIndex = 6;
          break;
        case PtColorSpace.XyY:
          comboBox2.SelectedIndex = 7;
          break;
        case PtColorSpace.XYZ:
          comboBox2.SelectedIndex = 8;
          break;
      }

      ConvertColor();

      checkBox1.Checked = Config.SmallWindow;   
      toolStripStatusLabel1.Text = Config.AppCaption + " (build " + Config.AppVersion + ")";
    }

    public void SetToolTipAndLabel()
    {
      String str1 = "";
      String str2 = "";
      String str3 = "";

      switch (comboBox1.SelectedIndex)
      {
        case 0:
          label_srcX.Text = "R";
          label_srcY.Text = "G";
          label_srcZ.Text = "B";

          str1 = "R [0..255]";
          str2 = "G [0..255]";
          str3 = "B [0..255]"; 
          break;

        case 1:
          label_srcX.Text = "C";
          label_srcY.Text = "M";
          label_srcZ.Text = "Y";

          str1 = "C [0.0..1.0]";
          str2 = "M [0.0..1.0]";
          str3 = "Y [0.0..1.0]";
          break;

        case 2:
          label_srcX.Text = "H";
          label_srcY.Text = "S";
          label_srcZ.Text = "V";

          str1 = "H [0.0..360.0]";
          str2 = "S [0.0..100.0]";
          str3 = "V [0.0..100.0]";
          break;

        case 3:
          label_srcX.Text = "(H) L";
          label_srcY.Text = "(H) a";
          label_srcZ.Text = "(H) b";

          str1 = "(H) L [0.0..100.0]";
          str2 = "(H) a [-72.52..104.92]";
          str3 = "(H) b [-190.92..58.90]";
          break;

        case 4:
          label_srcX.Text = "I1";
          label_srcY.Text = "I2";
          label_srcZ.Text = "I3";

          str1 = "I1 [0..255]";
          str2 = "I2 [-127.5..127.50]";
          str3 = "I3 [-127.5..127.50]";
          break;

        case 5:
          label_srcX.Text = "L";
          label_srcY.Text = "a";
          label_srcZ.Text = "b";

          str1 = "L [0.0..100.0]";
          str2 = "a [-86.18..98.25]";
          str3 = "b [-107.86..98.48]";
          break;

        case 6:
          label_srcX.Text = "L";
          label_srcY.Text = "u";
          label_srcZ.Text = "V";

          str1 = "L [0.0..100.0]";
          str2 = "u [-83.08..175.05]";
          str3 = "V [-134.11..107.40]";
          break;

        case 7:
          label_srcX.Text = "x";
          label_srcY.Text = "y";
          label_srcZ.Text = "Y";

          str1 = "x [0.0..0.64]";
          str2 = "y [0.0..0.60]";
          str3 = "Y [0.0..100.0]";
          break;

        case 8:
          label_srcX.Text = "X";
          label_srcY.Text = "Y";
          label_srcZ.Text = "Z";

          str1 = "X [0.0..95.05]";
          str2 = "Y [0.0..100.0]";
          str3 = "Z [0.0..108.9]";
          break;
      }

      toolTip_SrcX.SetToolTip(this.textBox_srcX, str1);
      toolTip_SrcY.SetToolTip(this.textBox_srcY, str2);
      toolTip_SrcZ.SetToolTip(this.textBox_srcZ, str3);
    }


    private void inputHandler(object sender, KeyPressEventArgs e)
    {
      if (!Char.IsDigit(e.KeyChar) && e.KeyChar != (char)44 &&
          e.KeyChar != (char)45 && e.KeyChar != (char)46 && 
          e.KeyChar != (char)8  && e.KeyChar != (char)13)
      {
        e.Handled = true;   // forget this input
      }
    }

    private void inputKeyUp(object sender, KeyEventArgs e)
    {
      SetToolTipAndLabel();
      ConvertColor();
    }

    private void FormMaster_Load(object sender, EventArgs e)
    {
      Config.RegIniRead();
      this.Text = Config.AppCaption;

      _screenBmp = new Bitmap(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height);
      _pb1Bmp = new Bitmap(pictureBox1.Width, pictureBox1.Height);
      Graphics g = Graphics.FromImage(_pb1Bmp);
      g.Clear(Color.Black);
      g.Dispose();
    }

    private void FormMaster_FormClosing(object sender, FormClosingEventArgs e)
    {
      Config.RegIniSave();
    }


    private void PickColor()
    {
      ColorDialog colorDialog = new ColorDialog();
      colorDialog.AllowFullOpen = true;
      colorDialog.ShowHelp = true;
      colorDialog.Color = Color.Black;

      // Update the text box color if the user clicks OK 
      if (colorDialog.ShowDialog() == DialogResult.OK)
      {
        SetColor(colorDialog.Color.R, colorDialog.Color.G, colorDialog.Color.B);
      }
    }


    public void SetColor(Byte r, Byte g, Byte b)
    {
      double outX = 0;
      double outY = 0;
      double outZ = 0;

      Byte byteR = r;
      Byte byteG = g;
      Byte byteB = b;


      // convert from rgb to target space
      switch (comboBox1.SelectedIndex)
      {
        case 0:           // ptColorSpace.RGB

          outX = (double)byteR;
          outY = (double)byteG;
          outZ = (double)byteB;
          break;

        case 1:           // ptColorSpace.CMY
          PtColorConversion.RgbToCMY(ref byteR, ref byteG, ref byteB,
                          out outX, out outY, out outZ);
          break;

        case 2:           // ptColorSpace.HSV
          PtColorConversion.RgbToHSV(ref byteR, ref byteG, ref byteB,
                          out outX, out outY, out outZ);
          break;

        case 3:           // ptColorSpace.HunterLAB 
          PtColorConversion.RgbToHunterLAB(ref byteR, ref byteG, ref byteB,
                            out outX, out outY, out outZ);
          break;

        case 4:           // ptColorSpace.I1I2I3   
          PtColorConversion.RgbToI1I2I3(ref byteR, ref byteG, ref byteB,
                                 out outX, out outY, out outZ);
          break;

        case 5:           // ptColorSpace.LAB
          PtColorConversion.RgbToLAB(ref byteR, ref byteG, ref byteB,
                             out outX, out outY, out outZ);
          break;

        case 6:           // ptColorSpace.LUV
          PtColorConversion.RgbToLUV(ref byteR, ref byteG, ref byteB,
                            out outX, out outY, out outZ);
          break;

        case 7:           // ptColorSpace.XyY
          PtColorConversion.RgbToXyY(ref byteR, ref byteG, ref byteB,
                                    out outX, out outY, out outZ);
          break;

        case 8:           // ptColorSpace.XYZ
          PtColorConversion.RgbToXYZ(ref byteR, ref byteG, ref byteB,
                             out outX, out outY, out outZ);
          break;
      }


      if (comboBox1.SelectedIndex == 0)
      {
        textBox_srcX.Text = outX.ToString("0");
        textBox_srcY.Text = outY.ToString("0");
        textBox_srcZ.Text = outZ.ToString("0");
      }
      else
      {
        textBox_srcX.Text = outX.ToString("0.000");
        textBox_srcY.Text = outY.ToString("0.000");
        textBox_srcZ.Text = outZ.ToString("0.000");
      }  

    }

    private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
    {
      PickColor();
      SetToolTipAndLabel();
      ConvertColor();
    }

    private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
    {
      this.Cursor = Cursors.Hand;
    }

    private void pictureBox1_MouseLeave(object sender, EventArgs e)
    {
      this.Cursor = Cursors.Default;
    }

    private void button1_Click(object sender, EventArgs e)
    {
      if (!checkBox1.Checked)
      {

        toolTip_SrcX.Active = false;
        toolTip_SrcY.Active = false;
        toolTip_SrcZ.Active = false;
        Application.DoEvents();

        timer1.Enabled = !timer1.Enabled;

        if (timer1.Enabled)
        {
          button1.Text = "Capture F1";
          textBox_srcX.Enabled = false;
          textBox_srcY.Enabled = false;
          textBox_srcZ.Enabled = false;
          checkBox1.Enabled = false;
        }
        else
        {
          button1.Text = "From Screen";
          textBox_srcX.Enabled = true;
          textBox_srcY.Enabled = true;
          textBox_srcZ.Enabled = true;
          checkBox1.Enabled = true;
        }
      }
      else
      {
        this.Hide();
        FormCapture formCapture = new FormCapture(this);
        formCapture.ShowDialog();
      } 

    }


    private void GetColorFromScreen()
    {
      Graphics g1 = Graphics.FromImage(_screenBmp);
      Graphics g2 = Graphics.FromImage(_pb1Bmp);

      try
      {
        g1.CopyFromScreen(new Point(0, 0), new Point(0, 0), _screenBmp.Size);

        _screenColor = _screenBmp.GetPixel(_globalMouseX, _globalMouseY);

        SetColor(_screenColor.R, _screenColor.G, _screenColor.B);
        SetToolTipAndLabel();
        ConvertColor();
      }
      catch
      {
      }

      g1.Dispose();
      g2.Dispose();
    }

    private void GetGlobalMouseCoords()
    {
      Point defPnt = new Point();
      GetCursorPos(ref defPnt);

      _globalMouseX = defPnt.X;
      _globalMouseY = defPnt.Y;
    }

    private void timer1_Tick(object sender, EventArgs e)
    {
      GetGlobalMouseCoords();
      GetColorFromScreen();
    }

    private void FormMaster_KeyUp(object sender, KeyEventArgs e)
    {
      if (!checkBox1.Checked && e.KeyCode == Keys.F1)
        button1_Click(sender, e);
    }

    private void FormMaster_Resize(object sender, EventArgs e)
    {
      if (!checkBox1.Checked)
      {
        timer1.Enabled = false;
        button1.Text = "Screen Color Picker";
        textBox_srcX.Enabled = true;
        textBox_srcY.Enabled = true;
        textBox_srcZ.Enabled = true;
      }
    }

    private void checkBox1_MouseUp(object sender, MouseEventArgs e)
    {
      Config.SmallWindow = checkBox1.Checked;
    }

  }

}